@@ -1,6 +1,7 @@
 from bleak_retry_connector import close_stale_connections_by_address
 
 from .const import (
+    AutoLockMode,
     ConnectionInfo,
     DoorStatus,
     LockInfo,
@@ -20,10 +21,11 @@ from .util import (
     unique_id_from_local_name_address,
 )
 
-__version__ = "3.0.0"
+__version__ = "3.1.0"
 
 __all__ = [
     "AuthError",
+    "AutoLockMode",
     "ConnectionInfo",
     "DisconnectedError",
     "DoorStatus",
@@ -33,12 +35,12 @@ __all__ = [
     "LockStatus",
     "PushLock",
     "ValidatedLockConfig",
-    "serial_to_local_name",
-    "local_name_to_serial",
-    "unique_id_from_device_adv",
-    "unique_id_from_local_name_address",
-    "local_name_is_unique",
     "YaleXSBLEDiscovery",
     "YaleXSBLEError",
     "close_stale_connections_by_address",
+    "local_name_is_unique",
+    "local_name_to_serial",
+    "serial_to_local_name",
+    "unique_id_from_device_adv",
+    "unique_id_from_local_name_address",
 ]
